#include "ImageLib.h"

#ifdef NEW_HEADER_NAMES
#include <iostream>
#include <iomanip>
using std::cout;
using std::endl;
using std::ios;
#else
#include <iostream.h>
#include <iomanip.h>
#endif

#include "floatImage.h"
#include "ucharImage.h"
#include "PGMFile.h"
#include "ViffFile.h"
#include "Arguments.h"
#include "WvltFilters.h"

void readerror(char *filename);
void writeerror(char *filename);

// Perform wavelet transform on PGM format image, or inverse wavelet
// transform on raw float image
int main(int argc, char *argv[])
{
  CLArg param[] = {{(char *)"ifile",(char *)"input_file"},
                   {(char *)"ofile",(char *)"output_file"},
                   {0, 0}};
  CLArg optpm[] = {{0,0}};
  CLArg flags[] = {{(char *)"lvl", (char *)"transform_levels"},
                   {(char *)"std", 0},   //Do standard decomposition
		   {(char *)"inv", 0},   //Do inverse transform
		   {(char *)"w", (char *)"width"},
		   {(char *)"h", (char *)"height"},
		   {0,0}};
  Arguments args(argc, argv, param, optpm, flags);

  unsigned int levels = 0;
  args.value((char *)"lvl", levels);
  Wavelet wavelet = Antonini9_7; //Note: using periodic extension

  bool inv = args.value((char *)"inv");
  if (inv)
    {
      //Unless specified, input raw float image for the inverse
      //transform is assumed to be 512 x 512 pixels
      unsigned int w = 512;
      args.value((char *)"w", w);
      unsigned int h = 512;
      args.value((char *)"h", h);
      floatImage tr(w,h,0.0);
      //Read raw image into tr
      if (!tr.raw_read(args.arg((char *)"ifile")))
	readerror(args.arg((char *)"ifile"));

      floatImage fi;
      if (args.value((char *)"std"))
	fi = wavelet.ifwt_std(tr, levels, levels); //Standard decomposition
      else
	fi = wavelet.ifwt_nstd(tr, levels); //Non-standard decomposition

      //Convert float image into an unsigned char image
      ucharImage uci = fi;
      //Save unsigned char image in PGM format
      if (!uci.save(PGMFile(args.arg((char *)"ofile"))))
	writeerror(args.arg((char *)"ofile"));
    }
  else
    {  
      ucharImage uci;
      //Load PGM format file
      if (!uci.load(PGMFile(args.arg((char *)"ifile"))))
	readerror(args.arg((char *)"ifile"));

      floatImage tr, fi = uci;
      if (args.value((char *)"std"))
	tr = wavelet.fwt_std(fi, levels, levels); //Standard decomposition
      else
	tr = wavelet.fwt_nstd(fi, levels); //Non-standard decomposition

      //Save raw float image
      if (!tr.raw_write(args.arg((char *)"ofile")))
	writeerror(args.arg((char *)"ofile"));
    }
  return 0;
}

void readerror(char *filename)
{
  cerr << "Error: could not load file " << filename << endl;
  exit(1);
}

void writeerror(char *filename)
{
  cerr << "Error: could not save file " << filename << endl;
  exit(1);
}
